#ifndef WIN_IMPL_BASE_HPP
#define WIN_IMPL_BASE_HPP

namespace DuiLib
{
#define DUI_DECLARE_CUSTOM_MSG_MAP() \
    virtual LRESULT HandleCustomMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

#define DUI_BEGIN_CUSTOM_MSG_MAP(theClass) \
    LRESULT theClass::HandleCustomMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) \
    { \
        LRESULT lResult = 0; \
        switch(uMsg){ \
            case (UINT)(-1): \
            { \
                ASSERT(FALSE); \
                bHandled = TRUE; \
                break; \
            }
        
#define DUI_END_CUSTOM_MSG_MAP() \
            default:\
                lResult = __super::HandleCustomMessage(uMsg, wParam, lParam, bHandled); \
        } \
        return lResult; \
    }

    //TODO: bHandled = TRUE; ?
#define ON_CUSTOM_MSG(uMsg, func) \
        case uMsg: \
        { \
            bHandled = TRUE; \
            lResult = func(uMsg, wParam, lParam, bHandled); \
            break; \
        }

#define BEGIN_CUSTOM_COMMAND_GROUP() \
        case WM_COMMAND: \
        { \

#define ON_CUSTOM_COMMAND_MSG(id, func) \
            if(id == LOWORD(wParam)){ \
                bHandled = TRUE; \
                lResult = func((UINT)HIWORD(wParam), (int)LOWORD(wParam), (HWND)lParam, bHandled); \
            }

#define ON_CUSTOM_COMMAND_RANGE_MSG(idFirst, idLast, func) \
            if(idFirst <= LOWORD(wParam) && LOWORD(wParam) <= idLast){ \
                bHandled = TRUE; \
                lResult = func((UINT)HIWORD(wParam), (int)LOWORD(wParam), (HWND)lParam, bHandled); \
            }

#define END_CUSTOM_COMMAND_GROUP() \
            break; \
        }


	enum UILIB_RESOURCETYPE
	{
		UILIB_FILE=1,				// Դļ
		UILIB_ZIP,						// Դzipѹ
		UILIB_RESOURCE,			// Դ
		UILIB_ZIPRESOURCE,	// Դzipѹ
	};

	class UILIB_API WindowImplBase
		: public CWindowWnd
		, public CNotifyPump
		, public INotifyUI
		, public IMessageFilterUI
		, public IDialogBuilderCallback
	{
	public:
		WindowImplBase(){m_bHasFocusCtrl = true;};
		virtual ~WindowImplBase(){};
		virtual void InitWindow(){};
		virtual void OnFinalMessage( HWND hWnd );
		virtual void Notify(TNotifyUI& msg);

		DUI_DECLARE_MESSAGE_MAP()
		virtual void OnClick(TNotifyUI& msg);

		inline void SetHasFocusCtrl(bool bNeedFocus);//{m_bHasFocusCtrl = bNeedFocus;};

	protected:
		virtual CDuiString GetSkinFolder() = 0;
		virtual CDuiString GetSkinFile() = 0;
		virtual LPCTSTR GetWindowClassName(void) const = 0 ;
		LRESULT ResponseDefaultKeyEvent(WPARAM wParam);

		CPaintManagerUI m_PaintManager;
		static LPBYTE m_lpResourceZIPBuffer;

	public:
		CPaintManagerUI* GetPaintManager() {return &m_PaintManager;};
        virtual BOOL IsDumpNotifyMsg() const;
		virtual UINT GetClassStyle() const;
		virtual UILIB_RESOURCETYPE GetResourceType() const;
		virtual CDuiString GetZIPFileName() const;
		virtual LPCTSTR GetResourceID() const;
		virtual CControlUI* CreateControl(LPCTSTR pstrClass);
		virtual LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM /*lParam*/, bool& /*bHandled*/);
		virtual LRESULT OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);

#if defined(WIN32) && !defined(UNDER_CE)
		virtual LRESULT OnNcActivate(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnNcCalcSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnNcPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
		virtual LRESULT OnNcHitTest(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnNcLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnGetMinMaxInfo(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnMouseWheel(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnMouseHover(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
#endif
		virtual LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnChar(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnSysCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LRESULT OnKeyDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnKillFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnLButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
		virtual LRESULT HandleMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);
		virtual LRESULT HandleCustomMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
		virtual LONG GetStyle();
	private:
		bool m_bHasFocusCtrl; 
		//it will make the window has a default focus.For most of windows, it is true; Some window, such as CDuiTrayPopWnd, it must be false.
	};
}

#endif // WIN_IMPL_BASE_HPP
